#version 150

uniform	sampler2D	tex_source;
uniform sampler2D	tex_view_position;

uniform vec2	u_step;

in	vec2	p_tex_coord;

out vec4	blur_result;

const	int		kernel_width = 5;
const	float	exponent = 0.06;

void main()
{
	float sigma = (kernel_width - 1.0) / 6.0;

	float fragment_depth = texture2D(tex_view_position, p_tex_coord).z;

	float weights = 0.0;
	vec4 blurred = vec4(0.0);

	for (float i = -(kernel_width - 1.0) / 2.0; i < (kernel_width - 1.0) / 2.0; i++) {
		vec2 step_coord = vec2(p_tex_coord.x - i * u_step.x, p_tex_coord.y - i * u_step.y);
		float geometric_weight = exp(-pow(i, 2) / (2 * pow(sigma, 2)));
		float sample_depth = texture2D(tex_view_position, step_coord).z;
		float photometric_weight = 1 / pow((1 + abs(fragment_depth - sample_depth)), exponent);

		weights += geometric_weight * photometric_weight;
		blurred += texture2D(tex_source, step_coord) * geometric_weight * photometric_weight;
	}

	blur_result = blurred / weights;
}