#version 150

// Material properties
uniform sampler2D material_texture_diffuse;		// Diffuse texture of material
uniform sampler2D material_texture_normal;		// Normal texture of material
uniform sampler2D material_texture_specular;	// Specular texture of material

uniform float	material_shininess;

// View properties
uniform vec2	near_far_plane;

// Geometry components
in	vec3	view_space_position;
in	vec3	view_space_normal;
in	vec3	world_space_position;

in	vec2	texture_coordinate;	// Texture coordinate

in	mat3	normal_matrix;

out vec4 out0;	// Albedo
out vec4 out1;	// Normal (xyz) and Depth (w)
out vec4 out2;	// View Position
out	vec4 out3;	// Ambient.rgb (rgb), Diffuse.r (a)
out vec4 out4;	// Diffuse.gb (rg), Specular.rg (ba)
out vec4 out5;  // Specular.b (r) and Emissive.rgb (gba)

void main()
{
	// Read information from texture
	vec4 albedo_texture		= texture2D(material_texture_diffuse, texture_coordinate);
	vec4 normal_texture		= texture2D(material_texture_normal, texture_coordinate);
	vec4 specular_texture	= texture2D(material_texture_specular, texture_coordinate);

	// Invert y-channel of normal_vector
	vec3 texture_normal = normal_texture.xyz; 
	texture_normal.y = -texture_normal.y;

	// Calculate normal vector and depth
	vec3 normal_map_normal = normalize(normal_matrix * (normal_texture.xyz * 2.0 - 1.0));

	vec3 normal_vector = normalize(view_space_normal - normal_map_normal);
	float linear_depth = (length(view_space_position.z) - near_far_plane.x) / (near_far_plane.y - near_far_plane.x);

	out0 = vec4(albedo_texture.rgb, specular_texture.r);
	out1 = vec4(normal_vector, linear_depth);
	out2 = vec4(view_space_position, material_shininess);
}