#version 150

// Light position
uniform vec3	light_position;		// Direction of light
uniform vec3	light_color;		// Color of light
uniform	int		light_type;			// Type of light (1 = point light, 2 = directional light)

uniform vec3	world_position;		// World space position

uniform	sampler2D	tex_albedo;
uniform	sampler2D	tex_out1;
uniform	sampler2D	tex_out2;
uniform	sampler2D	tex_out3;

uniform sampler2D	tex_normal_depth;
uniform sampler2D	tex_view_position;

in	vec2	p_tex_coord;

out vec4	out_color;

void main()
{
	    vec3 l = normalize(light_position - world_position.xyz);    // light vector
	
		// If it is a directional light source, the 
		// light direction is passed as the position vector
		if(light_type == 2)
			l = light_position;

	    vec3 v = normalize(view_vector);							// view vector
		vec3 n = normalize(normal_vector);							// normal vector
		vec3 h = normalize(l + v);									// half vector
		
		// Get texture information
		vec4 material_texture	= texture(tex_albedo, p_tex_coord);
		vec4 material_out1		= texture(tex_out1, p_tex_coord);
		vec4 material_out2		= texture(tex_out2, p_tex_coord);
		vec4 material_out3		= texture(tex_out3, p_tex_coord);
		
		vec3	material_ambient	= c_out1.rgb;
		vec3	material_diffuse	= vec3(c_out1.a, c_out2.r, c_out2.g);
		vec3	material_specular	= vec3(c_out2.b, c_out2.a, c_out3.r);
		vec3	material_emissive	= vec3(c_out3.gba);
		float	material_shininess	= material_texture.a;
		
		// Calculate the lighting
		vec3	c_emissive		= material_emissive;
		vec3	c_ambient		= material_ambient;
		vec3	c_diffuse		= clamp(dot(n, l), 0, 1) * material_diffuse.rgb;
		vec3	c_specular		= pow(clamp(dot(n, h), 0, 1), material_shininess) * material_specular;
		vec3	c_texture		= material_texture.rgb;

		// Compute final color
		vec3 color = (clamp(c_ambient + c_diffuse, 0, 1) * c_texture + c_specular) * c_light_color + c_emissive;

		// Final color (no alpha)
		out_color = vec4(color, 1);
}