#version 150

uniform sampler2D tex_normals;
uniform sampler2D tex_positions;
uniform sampler2D tex_random;

uniform vec4 u_screen_size;

const float g_random_size = 64.0;
const float g_sample_radius = 5.0;
const float g_intensity = 5.0;
const float g_scale = 2.0;
const float g_bias = 0.00002;

in	vec2 io_tex_coord;

out	vec4 out0;

vec3 samplePosition(vec2 uv) {
	return texture2D(tex_positions, uv).xyz;
}

vec3 sampleNormal(vec2 uv) {
	return texture2D(tex_normals, uv).xyz;
}

vec2 sampleRandom(vec2 uv) {
	vec2 rnd = texture2D(tex_random, u_screen_size.xy * uv / g_random_size).xy;
	return normalize(rnd * 2.0 - 1.0f);
}

float calculateOcclusion(vec2 tuv, vec2 uv, vec3 p, vec3 n) {
	vec3 delta = samplePosition(tuv + uv) - p;
	vec3 v = normalize(delta);
	float d = length(delta) * g_scale;

	return max(0.0, dot(n, v) - g_bias) * (1.0 / (1.0 + d)) * g_intensity;
}

void main() {
	const vec2 vec[4] = vec2[](vec2(1,0), vec2(-1,0), vec2(0,1), vec2(0,-1));

	vec3 p = samplePosition(io_tex_coord);
	vec3 n = sampleNormal(io_tex_coord);
	vec2 rand = sampleRandom(io_tex_coord);

	float ao = 0.0f;
	float rad = g_sample_radius / p.z;

	int iterations = 4;
	for (int j = 0; j < iterations; j++) {
		vec2 c1 = reflect(vec[j], rand) * rad;
		vec2 c2 = vec2(c1.x * 0.707 - c1.y * 0.707, c1.x * 0.707 + c1.y * 0.707);
		
		ao += calculateOcclusion(io_tex_coord, c1 * 0.25, p, n);
		ao += calculateOcclusion(io_tex_coord, c2 * 0.5, p, n);
		ao += calculateOcclusion(io_tex_coord, c1 * 0.75, p, n);
		ao += calculateOcclusion(io_tex_coord, c2, p, n);
	}

	ao /= float(iterations) * 4.0;

	out0 = vec4(ao);
}