#version 150

uniform sampler2D tex_albedo_specular;	// Albedo (rgb) and specular intensity (a)
uniform sampler2D tex_view_positions;	// View position (xyz) and specular power (a) texture
uniform sampler2D tex_normal_depth;		// Normal (xyz) and depth (w) texture

uniform vec4	screen_dimension;		// Screen dimensions: width, height, 1/width, 1/height

uniform mat4	inverse_vp_matrix;

uniform vec3	view_position;			// Position of the camera

uniform float	light_radius;			// Radius of the light
uniform float	light_intensity;		// Brightness of the light
uniform	vec3	light_color;			// Color of the light

in	vec4 screen_position;
in  vec3 light_position_view;

out	vec4 out0;

void main()
{
	// Perspective division of screen position
	vec4 screen_pos = screen_position / screen_position.w;

	// Align screen coordinates from [-1, 1] range to [0, 1] range
	vec2 tex_coord = 0.5 * (vec2(screen_pos.x, screen_pos.y) + 1.0);

	// Align texels to pixels
	tex_coord -= screen_dimension.zw / 2.0;

	// Get normal and depth data from normal map
	vec4 normal_depth = texture2D(tex_normal_depth, tex_coord);
	vec3 normal_vector = normal_depth.xyz;
	float depth = normal_depth.w;
	
	vec4 view_position_specular_power = texture2D(tex_view_positions, tex_coord);

	vec3 position = view_position_specular_power.xyz;

	// Compute light vector
	vec3 light_vector = light_position_view - position.xyz;
	
	// Compute attenuation based on distance - linear attenuation
	float attenuation = clamp(1.0 - length(light_vector)/light_radius, 0.0, 1.0);

	// Normalize light vector
	light_vector = normalize(light_vector);

	// Compute diffuse light
	float NdotL = max(0, dot(normal_vector, light_vector));
	vec3 diffuse_light = NdotL * light_color;

	// Reflection vector
	vec3 reflection_vector = normalize(reflect(-light_vector, normal_vector));

	// Camera-to-surface vector
	vec3 camera_direction = normalize(view_position - position.xyz);

	// Get specular data
	float specular_intensity = texture2D(tex_albedo_specular, tex_coord).a;
	float specular_power = view_position_specular_power.a;

	// Compute specular light
	float specular_light = specular_intensity * pow(clamp(dot(reflection_vector, camera_direction), 0.0, 1.0), specular_power);

	out0 = attenuation * light_intensity * vec4(diffuse_light, specular_light);
}