#version 150

// Light position
uniform vec3	light_position;		// Direction of light
uniform vec3	light_color;		// Color of light
uniform	int		light_type;			// Type of light (1 = point light, 2 = directional light)

// Material properties
uniform vec3	material_emissive;	// Emissive color of material
uniform vec3	material_ambient;	// Ambient color of material
uniform vec4	material_diffuse;	// Diffuse color of material including ALPHA CHANNEL
uniform vec3	material_specular;	// Specular color of material
uniform float	material_shininess;	// Shininess of material

uniform sampler2D material_texture_diffuse;	// Diffuse texture of material

// Geometry components
in  vec4	world_position;		// Position in world space
in  vec3	normal_vector;		// Surface normal
in	vec3	view_vector;		// View vector
in	vec2	texture_coordinate;	// Texture coordinate

out vec4 out_color;

void main()
{
    vec3 l = normalize(light_position - world_position.xyz);    // light vector

	// If it is a directional light source, the 
	// light direction is passed as the position vector
	if(light_type == 2)
		l = light_position;

    vec3 v = normalize(view_vector);							// view vector
	vec3 n = normalize(normal_vector);							// normal vector
    vec3 h = normalize(l + v);									// half vector
	
	// Calculate the emissive lighting
	vec3 c_emissive = material_emissive;

	// Calculate the ambient reflection
	vec3 c_ambient = material_ambient;

	// Calculate the diffuse reflection
	vec3 c_diffuse = clamp(dot(n, l), 0, 1) * material_diffuse.rgb;

	// Calculate the specular reflection
	vec3 c_specular = pow(clamp(dot(n, h), 0, 1), material_shininess) * material_specular;

	// Sample the texture
	vec4 c_texture = texture2D(material_texture_diffuse, texture_coordinate);

	// Combine all color components
	vec3 color = (clamp(c_ambient + c_diffuse, 0, 1) * c_texture.rgb + c_specular) * light_color + c_emissive;
	
	// Calculate the transparency
	float alpha = material_diffuse.a * c_texture.a;

	// Return the pixel color
	out_color = vec4(vec3(texture_coordinate, 0), alpha);
}