#version 150

// Material properties
uniform vec3	material_emissive;	// Emissive color of material
uniform vec3	material_ambient;	// Ambient color of material
uniform vec4	material_diffuse;	// Diffuse color of material including ALPHA CHANNEL
uniform vec3	material_specular;	// Specular color of material
uniform float	material_shininess;	// Shininess of material

uniform sampler2D material_texture_diffuse;	// Diffuse texture of material

// View properties
uniform vec2	near_far_plane;

// Geometry components
in	vec3	view_space_position;
in	vec3	view_space_normal;

in	vec2	texture_coordinate;	// Texture coordinate

out vec4 out0;	// Albedo
out vec4 out1;	// Normal (xyz) and Depth (w)
out vec4 out2;	// View Position
out	vec4 out3;	// Ambient.rgb (rgb), Diffuse.r (a)
out vec4 out4;	// Diffuse.gb (rg), Specular.rg (ba)
out vec4 out5;  // Specular.b (r) and Emissive.rgb (gba)

void main()
{
	float linearDepth = (length(view_space_position.z) - near_far_plane.x) / (near_far_plane.y - near_far_plane.x);

	// Write albedo texture
	out0 = vec4(c_texture.rgb, material_shininess);

	// Write depth and normal (maybe add normal mapping here)
	out1 = vec4(normalize(view_space_normal), linearDepth);

	// Write view positions (we have one value for free)
	out2 = vec4(view_space_position, 0);

	// Write shading information into 5 shaders
	out3 = vec4(material_ambient.r, material_ambient.g, material_ambient.b, material_diffuse.r);
	out4 = vec4(material_diffuse.g, material_diffuse.b, material_specular.r, material_specular.b);
	out5 = vec4(material_specular.b, material_emissive.r, material_emissive.g, material_emissive.b);
}